import { Component, OnInit, Inject } from '@angular/core';
import { HttpClient, HttpHeaders, HttpResponse } from '@angular/common/http';
import { ActivatedRoute, ParamMap, Router } from '@angular/router';
import * as $ from "jquery";
 
export interface Project {
  id: string;
  organisationId: string;
  organisationName: string;
  name: string;
  isActive: boolean;
}

export interface Organisation {
  id: string;
  slug: string;
  name: string;
  logoUrl: string;
}

export interface Folder {
  id: string;
  name: string;
}

export interface Document {
  id: string;
  name: string;
  currentVersionId: string;
  folderName: string;
  lastUpdated: Date;
  fileNameWithExtension: string;
  fileSizeString: string;
  revision: string;
}

export interface ViewModel {
  currentVersionId: string;
  name: string;
  folderName: string;
  lastUpdated: Date | null;
  fileNameWithExtension: string;
  fileSizeString: string;
  revision: string;
}

@Component({
  selector: 'app-document',
  templateUrl: './document.component.html',
  styleUrls: ['./document.component.css']
})
export class DocumentComponent implements OnInit {
  private http: HttpClient;
  private baseUrl: string;
  private accessToken: string;
  private projectId: string;
  private folderId: string;
  private documentId: string;
  public projects: Project[];
  public organisations: Organisation[];
  public folders: Folder[];
  public documents: Document[];
  public documentRevison: boolean;

  public isUploadDuplicateFile: boolean;
  public uploadedFileName: string;
  public uploadDone : boolean;
  public uploadFile;

  DisableUpload(): boolean {
    let filePathInput: any = $("#filePath");
   
    return this.folderId == null || filePathInput == null || (filePathInput[0].files[0] == null);
  }

  DisableDownload(): boolean {
    return this.documentId == null;
  }

  private orgSlug: string;

  public viewModel: ViewModel;

  constructor(
    httpclient: HttpClient,
    @Inject('BASE_URL') url: string,
    private route: ActivatedRoute)
  {
    this.baseUrl = url;
    this.http = httpclient;
    this.viewModel = <ViewModel>{ currentVersionId: "", name: "", folderName: "", lastUpdated: null, fileNameWithExtension: "", fileSizeString: "" };
  }

  ngOnInit() {
    this.organisations = [];
    this.isUploadDuplicateFile = false;
    this.uploadDone = false;
    this.uploadedFileName = "";
    this.documentRevison = true;
    this.accessToken = this.route.snapshot.paramMap.get('token');
    this.http.get<string>(this.baseUrl + 'api/SampleData/GetOrganisations?accessToken=' + this.accessToken).subscribe(result => {
      JSON.parse(result).forEach(org => this.organisations.push(org));
      console.log("return organisations " + this.organisations);
    }, error => {
      console.log("error:" + error)
    }
    );
  }

  public fetchProjects(event) {
    this.resetDocument();
    this.projectId = null;
    if (this.accessToken) {
      this.projects = [];
      this.orgSlug = event.value;
      this.http.get<string>(this.baseUrl + 'api/SampleData/GetProjects?slug=' + event.value + '&accessToken=' + this.accessToken).subscribe(result => {
        JSON.parse(result).items.forEach(p => this.projects.push(p));
      }, error => {
          console.log("error: "  + error);
      });
    }
  }

  public fetchFolders(event) {
    this.folderId = null;
    this.documents = [];
    this.resetDocument();
    if (this.accessToken) {
      this.folders = [];
      this.projectId = event.value;
      this.http.get<string>(this.baseUrl + 'api/SampleData/GetFolders?slug=' + this.orgSlug + '&projectId=' + event.value + '&accessToken=' + this.accessToken).subscribe(result => {
        JSON.parse(result).data.forEach(p => this.folders.push(p));
      }, error => {
        console.log("error: " + error);
      });
    }
  }

  public fetchDocuments(event) {
    this.resetDocument();
    if (this.accessToken) {
      this.documents = [];
      this.folderId = event.value;
      this.http.get<string>(this.baseUrl + 'api/SampleData/GetDocuments?slug=' + this.orgSlug + "&projectId=" + this.projectId + "&folderId=" + event.value + '&accessToken=' + this.accessToken).subscribe(result => {
        JSON.parse(result).data.pagedData.items.forEach(d => this.documents.push(d));
      }, error => {
        console.log("error: " + error);
      });
    }
  }

  public onDocumentChange(event) {
    this.resetDocument();
    this.documentId = event.value;
    let document = this.documents.find(d => d.id === event.value);

    this.viewModel = { ...document };

    console.log('document id ', this.documentId);
  }

  public downloadDocument() {
    if (this.accessToken && this.viewModel && this.documentId) {
      let localApi = "api/v2/organisation/" + this.orgSlug + "/Documents/DownloadDocument?documentId=" + this.documentId;
      let customheaders: HttpHeaders;
      //below is local accessKey for local debug only
      //let accessKey = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJlbWFpbCI6InBlbmcuZGFpQHRvdGFsc3luZXJneS5jb20iLCJhcHBsaWNhdGlvbmlkIjowLCJ1c2VybmFtZSI6InBlbmdkYWkiLCJ0ZW5hbnQiOiJkZXZlbG9wZXIiLCJjb2RlX2V4cGlyZXMiOjYzNzMyMjc4OTc1MDUzMDAyNH0.MIcTvdu8en_P4bH_BFJE6fNVPl2rHOGM7AbnHMr7PDs";
      //customheaders = new HttpHeaders({ "access-token": accessKey });
      customheaders = new HttpHeaders({ "access-token": this.accessToken });
      const httpOptions = {
        headers: customheaders        
      };

      this.http.get(
        //"http://localhost:5000/" + localApi, { ...httpOptions, responseType: 'arraybuffer',})
        this.baseUrl + 'api/SampleData/DownloadDocument?slug=' + this.orgSlug + "&id=" + this.documentId + '&accessToken=' + this.accessToken,
        { ...httpOptions, responseType : 'blob' })
        .subscribe((result) => {
          let linkElement = document.createElement('a');
          try {
            let blob = new Blob([result], { type: this.viewModel.fileNameWithExtension.split('.').slice(-1)[0]});
            let url = window.URL.createObjectURL(blob);

            linkElement.setAttribute('href', url);
            linkElement.setAttribute("download", this.viewModel.fileNameWithExtension);

            var clickEvent = new MouseEvent("click", {
              "view": window,
              "bubbles": true,
              "cancelable": false
            });
            linkElement.dispatchEvent(clickEvent);
          }
          catch (ex) {
            console.log(ex);
          }
        }, error => {
          console.log("error: " + error);
        });
    }
  }

  public uploadDocument() {
    if (this.accessToken) {
      this.uploadedFileName = "";
      let filePathInput: any = $("#filePath");
      if (filePathInput[0].files) {
        this.uploadFile = filePathInput[0].files[0];

        const formData = new FormData();
        formData.append(this.uploadFile.name, this.uploadFile);

        this.http.post<any>(
          this.baseUrl + 'api/SampleData/UploadDocument?slug=' + this.orgSlug + '&revision=' + this.documentRevison + '&projectId=' + this.projectId + '&folderId=' + this.folderId + '&accessToken=' + this.accessToken,
          formData)
          .subscribe((result) => {

            result.headers.forEach(function (item, index) {

              if (item.key === 'uploadFileName') {
                this.uploadedFileName = item.value;
              }

              if (item.key === 'isFileDuplicate') {
                this.isUploadDuplicateFile = item.value[0] === 'True';
              }
            }.bind(this));

            this.uploadDone = true;
        }, error => {
          console.log("error: " + error);
        });
      }
    }
  }

  private resetDocument() {
    this.documentId = null;
    this.isUploadDuplicateFile = false;
    this.uploadedFileName = "";
    this.documentRevison = true;
    this.uploadDone = false;
    this.viewModel = <ViewModel>
      {
        currentVersionId: "",
        name: "", folderName: "",
        lastUpdated: null,
        fileNameWithExtension: "",
        fileSizeString: "",
        revision:"",
      };
  }
}
